/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this file,
 * You can obtain one at http://mozilla.org/MPL/2.0/. */

"use strict";

const Cc = Components.classes;
const Ci = Components.interfaces;
const Cu = Components.utils;

Cu.import("resource://gre/modules/XPCOMUtils.jsm");
Cu.import("resource://gre/modules/Services.jsm");
Cu.import("resource://gre/modules/FileUtils.jsm");

function ProfileMigrator() {
}

ProfileMigrator.prototype = {
  migrate: function PM_migrate(aStartup, aKey) {
    // By opening the wizard with a supplied migrator, it will automatically
    // migrate from it.
    let key = null, migrator = null;
    let skipImportSourcePage = Cc["@mozilla.org/supports-PRBool;1"]
                                 .createInstance(Ci.nsISupportsPRBool);
    if (aKey) {
      key = aKey;
      migrator = this._getMigratorIfSourceExists(key);
      if (!migrator) {
        Cu.reportError("Invalid migrator key specified or source does not exist.");
        return;
      }
      // If the migrator was passed to us from the caller, use that migrator
      // and skip the import source page.
      skipImportSourcePage.data = true;
    } else {
      [key, migrator] = this._getDefaultMigrator();
    }
    if (!key)
        return;

    let params = Cc["@mozilla.org/array;1"].createInstance(Ci.nsIMutableArray);
    params.appendElement(this._toCString(key), false);
    params.appendElement(migrator, false);
    params.appendElement(aStartup, false);
    params.appendElement(skipImportSourcePage, false);

    Services.ww.openWindow(null,
                           "chrome://browser/content/migration/migration.xul",
                           "_blank",
                           "chrome,dialog,modal,centerscreen,titlebar",
                           params);
  },

  canMigrate: function PM__canMigrate(aMigratorKey) {
    return "@mozilla.org/profile/migrator;1?app=browser&type=" + aMigratorKey in Cc;
  },

  _toCString: function PM__toCString(aStr) {
    let cstr = Cc["@mozilla.org/supports-cstring;1"].
               createInstance(Ci.nsISupportsCString);
    cstr.data = aStr;
    return cstr;
  },

  _getMigratorIfSourceExists: function PM__getMigratorIfSourceExists(aKey) {
    try {
      let cid = "@mozilla.org/profile/migrator;1?app=browser&type=" + aKey;
      let migrator = Cc[cid].createInstance(Ci.nsIBrowserProfileMigrator);
      if (migrator.sourceExists)
        return migrator;
    } catch (ex) {
      Cu.reportError("Could not get migrator: " + ex);
    }
    return null;
  },

  // We don't yet support checking for the default browser on all platforms,
  // needless to say we don't have migrators for all browsers.  Thus, for each
  // platform, there's a fallback list of migrators used in these cases.
  _PLATFORM_FALLBACK_LIST:
//@line 86 "/linux/repository/dists/squeeze/x86_64/firefox/iceweasel-13.0.1/browser/components/migration/src/ProfileMigrator.js"
     ["chrome"],
//@line 88 "/linux/repository/dists/squeeze/x86_64/firefox/iceweasel-13.0.1/browser/components/migration/src/ProfileMigrator.js"

  _getDefaultMigrator: function PM__getDefaultMigrator() {
    let migratorsOrdered = Array.slice(this._PLATFORM_FALLBACK_LIST);
    let defaultBrowser = "";
//@line 143 "/linux/repository/dists/squeeze/x86_64/firefox/iceweasel-13.0.1/browser/components/migration/src/ProfileMigrator.js"

    // If we found the default browser and we have support for that browser,
    // make sure to check it before any other browser, by moving it to the head
    // of the array.
    if (defaultBrowser)
      migratorsOrdered.sort(function(a, b) b == defaultBrowser ? 1 : 0);

    for (let i = 0; i < migratorsOrdered.length; i++) {
      let migrator = this._getMigratorIfSourceExists(migratorsOrdered[i]);
      if (migrator)
        return [migratorsOrdered[i], migrator];
    }

    return ["", null];
  },

  QueryInterface: XPCOMUtils.generateQI([Ci.nsIProfileMigrator]),
  classDescription: "Profile Migrator",
  contractID: "@mozilla.org/toolkit/profile-migrator;1",
  classID: Components.ID("6F8BB968-C14F-4D6F-9733-6C6737B35DCE")
};

let NSGetFactory = XPCOMUtils.generateNSGetFactory([ProfileMigrator]);
